aux.ahaz = function(outcome, time1, time2=NULL, tau, tau0=0, X, Z){
  
  if(!is.null(time2)){
    etime = time2 - time1
    otime = sort(time2[outcome==1]); otime1 = c(otime[1]-.1, otime); dtime = diff(otime1)
    Yt = t(mapply(function(x, y){as.numeric(otime > x & otime <= y)}, x = time1, y = time2))
  }
  
  if(is.null(time2)){
    etime = time1;
    otime = sort(etime[outcome==1]); otime1 = c(0, otime); dtime = diff(otime1)
    Yt = t(mapply(function(x) { as.numeric(otime <= x) }, x = etime))
  }
  
  n = nrow(Yt); nt = ncol(Yt)

  Xt = array(NA, dim = c(n, nt, ncol(X)))
  Zt = array(NA, dim = c(n, nt, ncol(Z)))
  for(j in 1:ncol(X)){Xt[,,j] = sweep(Yt, 1, X[,j], "*")}
  for(j in 1:ncol(Z)){Zt[,,j] = sweep(Yt, 1, Z[,j], "*")}
  
  evnt = apply(sweep(Yt, 1, outcome, "*"), 1, function(x){ifelse(all(x==0), 0, max(which(x==1)))})
  
  dNt = matrix(0, n, nt); dNt[cbind(1:n, evnt)] = 1; dNt = sweep(dNt, 1, outcome, "*")
  
  XX = simplify2array(mapply(function(k){crossprod(cbind(Yt[,k], Xt[,k,]))}, k=1:nt, SIMPLIFY = F))
  ZZ = simplify2array(mapply(function(k){crossprod(Zt[,k,])}, k=1:nt, SIMPLIFY = F))
  XZ = simplify2array(mapply(function(k){crossprod(cbind(Yt[,k], Xt[,k,]), Zt[,k,])}, k = 1:nt, SIMPLIFY = F))
  XZdt = sweep(XZ, 3, dtime, "*")
  ZHZ = simplify2array(mapply(function(k){ZZ[,,k] - (crossprod(XZ[,,k], ginv(XX[,,k])) %*% XZ[,,k])}, k=1:nt, SIMPLIFY = F))
  ZdN = simplify2array(mapply(function(k){crossprod(Zt[,k,], dNt[,k,drop=F])}, k=1:nt, SIMPLIFY = F))
  XdN = simplify2array(mapply(function(k){crossprod(cbind(Yt[,k], Xt[,k,]), dNt[,k,drop=F])}, k=1:nt, SIMPLIFY = F))
  ZHdN = simplify2array(mapply(function(k){ZdN[,,k] - (crossprod(XZ[,,k], ginv(XX[,,k])) %*% XdN[,,k])}, k=1:nt, SIMPLIFY = F))
  
  intZHZ = apply(sweep(ZHZ, 3, dtime, "*"), c(1,2), sum)
  intZHdN = apply(ZHdN, c(1,2), sum)
  
  #### ESTIMATES ####
  gam.hat = solve(intZHZ, intZHdN)
  
  dBt.hat = t(mapply(function(k){ginv(XX[,,k]) %*% (XdN[,,k] - XZdt[,,k] %*% gam.hat)}, k=1:nt, SIMPLIFY = T))
  Bt.hat = cbind(time = otime, apply(dBt.hat, 2, cumsum))
  
  id.tau = max(which(otime < tau0 + tau))
  id.tau0 = ifelse(tau0!=0, max(which(otime < tau0)), 1)
  
  Bt.hat.tau = Bt.hat[id.tau, ] - Bt.hat[id.tau0,]
  est = c(Bt.hat.tau = Bt.hat.tau[-1], gam.hat = gam.hat)
  
  #### MARTINGALE RESIDUALS ####
  XdBt = mapply(function(k){cbind(Yt[,k], Xt[,k,]) %*% dBt.hat[k,]}, k=1:nt, SIMPLIFY = T) 
  Zgamdt = mapply(function(k){Zt[,k,] %*% gam.hat * dtime[k]}, k = 1:nt)
  
  dMt = dNt - XdBt - Zgamdt
  
  #### INFLUENCE FUNCTIONS ####
  ZHt = simplify2array(mapply(function(k){crossprod(tcrossprod(ginv(XX[,,k]), cbind(Yt[,k], Xt[,k,])), XZ[,,k])},
                              k = 1:nt, SIMPLIFY = F))
  intZHdMi = apply(simplify2array(mapply(function(k){sweep(Zt[,k,]-ZHt[,,k], 1, dMt[,k], "*")}, k = 1:nt, SIMPLIFY = F)), c(1,2), sum)
  
  if.gam = if.gam.copy = intZHdMi %*% ginv(intZHZ)
  
  XdM = simplify2array(mapply(function(k){sweep(cbind(Yt[,k], Xt[,k,]), 1, dMt[,k], "*")}, k=1:nt, SIMPLIFY = F))
  res.dBt.gam = simplify2array(mapply(function(k){tcrossprod(if.gam, XZdt[,,k])}, k = 1:nt, SIMPLIFY = F))
  if0.dBt = simplify2array(mapply(function(k){XdM[,,k] %*% ginv(XX[,,k])}, k=1:nt, SIMPLIFY = F))
  if1.dBt.gam = simplify2array(mapply(function(k){res.dBt.gam[,,k] %*% ginv(XX[,,k])}, k = 1:nt, SIMPLIFY = F))
  if.dBt = simplify2array(mapply(function(k){(XdM[,,k]-res.dBt.gam[,,k]) %*% ginv(XX[,,k])}, k=1:nt, SIMPLIFY = F))
  
  aux = cbind(apply((XdM-res.dBt.gam)[,,id.tau0:id.tau], c(1,2), sum), intZHdMi)
  colnames(aux) = c(paste("Beta:", c(0, colnames(X, do.NULL = F, prefix = "X")), sep = ""),
                    paste("gamma:", colnames(Z, do.NULL = F, prefix = "Z"), sep = ""))

  return(aux)
}
